package edu.sdsc.inca.depot.util;

import edu.sdsc.inca.util.StringMethods;

/**
 * A Notifier that sends email to its target.
 *
 * @author jhayes
 */
public class EmailNotifier implements Notifier {

  /**
   * Emails a target email address about an AcceptedOutput state change.
   *
   * @param comparitor the name of the class that did the test
   * @param comparison the test that changed state
   * @param reporter the name of the reporter generating the output
   * @param resource the resource where the reporter ran
   * @param target The email address notify
   * @param state A string indicating the new AO state
   */
  public void notify(String comparitor, String comparison,
                     String reporter, String resource,
                     String target, String state) {
    String messageBody =
      "This is notification from the Inca system that results of the test\n" +
      comparison + "\n" +
      "on the output of " + reporter + " on " + resource + " have changed.\n" +
      "Comparitor message: " + state + "\n";
    String messageSubject = "Inca Notification";
    StringMethods.sendEmail(target, messageSubject, messageBody);
  }

}
