package edu.sdsc.inca.depot.util;

import edu.sdsc.inca.protocol.MessageHandler;
import edu.sdsc.inca.protocol.ProtocolException;
import edu.sdsc.inca.protocol.ProtocolReader;
import edu.sdsc.inca.protocol.ProtocolWriter;
import edu.sdsc.inca.depot.persistent.HibernateUtil;
import org.hibernate.Session;

/**
 * Created by IntelliJ IDEA.
 *
 * @author Shava Smallen &lt;ssmallen@sdsc.edu&gt;
 */
abstract public class HibernateMessageHandler extends MessageHandler {
  Session session = null;


  /**
   * Overrides regular MessageHandler execute function in order to handle
   * hibernate sessions effectively. Will open a hibenate session and then
   * ensure that it's closed after a task is executed.  Caller will need to
   * create a function called executeHibernateAction.
   *
   * @throws Exception
   */
  public void execute(ProtocolReader reader,
                      ProtocolWriter writer) throws Exception {

    try {
      HibernateUtil.getCurrentSession();
      executeHibernateAction(reader,writer);
    } finally {
      HibernateUtil.closeSession();
    }
  }

  /**
   * Execute a hibernate-related task efficiently (i.e., session is safely
   * opened and closed.
   *
   * @throws ProtocolException
   */
  public abstract void executeHibernateAction(
      ProtocolReader reader, ProtocolWriter writer) throws Exception;
}
